//****************************************************************************
//
// File         : clip.cpp
//
// Description  : Polygon clipping functions.
//                Written by Lars Andersson (lars_and@algonet.se), 1998
//
//****************************************************************************

#include "main.h"
#include "fixed.h"

/*---------------------------------------------------------------------------
    Reset clipping window to screen boundaries
---------------------------------------------------------------------------*/

void Reset_Clip(Sector *sector_p)
{
    sector_p->clip.top_clip = engine.screen.top_clip;
    sector_p->clip.left_clip = engine.screen.left_clip;
    sector_p->clip.right_clip = engine.screen.right_clip;
    sector_p->clip.bottom_clip = engine.screen.bottom_clip;

    for (int i = (int)engine.screen.top_clip; i < (int)engine.screen.bottom_clip; i++) {
        sector_p->clip.hclip[i].sx = 0;
        sector_p->clip.hclip[i].ex = (int)(engine.screen.right_clip) << FIXED_POINT;
    }
}

/*---------------------------------------------------------------------------
    Z-Clip unshaded polygon
---------------------------------------------------------------------------*/

int ZClip_FlatPolygon(Vertice_2DD *xy, Polygon *poly)
{
    int             v1, v2, c;
    double          d, x, y;
    short           *index;
    Vertice_3D      *tp;

    index = poly->pindex;
    tp = engine.transformed_p;

    c = 0;

    for (v1 = 0; v1 < poly->nr_vertices; v1++) {
        v2 = v1 + 1;
        if (v2 == poly->nr_vertices) v2 = 0;

        if (tp[index[v1]].z >= NEAR_CLIP) {
            xy[c++] = engine.projected_p[index[v1]];
            if (tp[index[v2]].z >= NEAR_CLIP) continue;
        } else
            if (tp[index[v2]].z < NEAR_CLIP) continue;

        d = (NEAR_CLIP - tp[index[v1]].z) /
            (tp[index[v2]].z - tp[index[v1]].z);

        x = tp[index[v1]].x + d * (tp[index[v2]].x - tp[index[v1]].x);
        y = tp[index[v1]].y + d * (tp[index[v2]].y - tp[index[v1]].y);

        xy[c].x = (double)engine.screen.xcenter + x * engine.screen.pmulx_zclip;
        xy[c].y = (double)engine.screen.ycenter + y * engine.screen.pmuly_zclip;
        c++;
    }
    return(c);
}

/*---------------------------------------------------------------------------
    Clip unshaded polygon to clip window
---------------------------------------------------------------------------*/

int WClip_FlatPolygon(Vertice_2DD *xy, int corners)
{

    int             v1, v2, c;
    double          minx, maxx, miny, maxy, d;
    Vertice_2DD     *src, *dst, *tmp, clip[VERTICES_PER_POLYGON * 2];
    Clip            *box = engine.screen.cclip;

    // Do bounding box test

    minx = maxx = xy[0].x;
    miny = maxy = xy[0].y;
 
    for (v1 = 1; v1 < corners; v1++) {
       if (xy[v1].x < minx)
          minx = xy[v1].x;

       if (xy[v1].x > maxx)
          maxx = xy[v1].x;
 
       if (xy[v1].y < miny)
          miny = xy[v1].y;

       if (xy[v1].y > maxy)
          maxy = xy[v1].y;
    }

    // Completely outside?

    if (maxx < box->left_clip || minx > box->right_clip ||
        maxy < box->top_clip || miny > box->bottom_clip) return(0);

    // Completely inside?

    if (minx > box->left_clip && maxx < box->right_clip &&
        miny > box->top_clip && maxy < box->bottom_clip) return(corners);


    // Clip polygon to bounding box

    c = 0;
    src = xy;
    dst = clip;

    for (v1 = 0; v1 < corners; v1++) {
        v2 = v1 + 1;
        if (v2 == corners) v2 = 0;

        if (src[v1].x >= box->left_clip) {
            dst[c++] = src[v1];
            if (src[v2].x >= box->left_clip) continue;
        } else
            if (src[v2].x < box->left_clip) continue;

        d = (box->left_clip - src[v1].x) / (src[v2].x - src[v1].x);

        dst[c].x = box->left_clip;
        dst[c].y = src[v1].y + d * (src[v2].y - src[v1].y);
        c++;
    }
    if (c < 3) return(0);
    corners = c;
    tmp = src; src = dst; dst = tmp;

    c = 0;
    for (v1 = 0; v1 < corners; v1++) {
        v2 = v1 + 1;
        if (v2 == corners) v2 = 0;

        if (src[v1].x <= box->right_clip) {
            dst[c++] = src[v1];
            if (src[v2].x <= box->right_clip) continue;
        } else {
            if (src[v2].x > box->right_clip) continue;
        }

        d = (box->right_clip - src[v1].x) / (src[v2].x - src[v1].x);

        dst[c].x = box->right_clip;
        dst[c].y = src[v1].y + d * (src[v2].y - src[v1].y);
        c++;
    }
    if (c < 3) return(0);
    corners = c;
    tmp = src; src = dst; dst = tmp;
 
    c = 0;
    for (v1 = 0; v1 < corners; v1++) {
        v2 = v1 + 1;
        if (v2 == corners) v2 = 0;

        if (src[v1].y <= box->bottom_clip) {
            dst[c++] = src[v1];
            if (src[v2].y <= box->bottom_clip) continue;
        } else {
            if (src[v2].y > box->bottom_clip) continue;
        }

        d = (box->bottom_clip - src[v1].y) / (src[v2].y - src[v1].y);

        dst[c].y = box->bottom_clip;
        dst[c].x = src[v1].x + d * (src[v2].x - src[v1].x);
        c++;
    }
    if (c < 3) return(0);
    corners = c;
    tmp = src; src = dst; dst = tmp;
 
    c = 0;
    for (v1 = 0; v1 < corners; v1++) {
        v2 = v1 + 1;
        if (v2 == corners) v2 = 0;

        if (src[v1].y >= box->top_clip) {
            dst[c++] = src[v1];
            if (src[v2].y >= box->top_clip) continue;
        } else {
            if (src[v2].y < box->top_clip) continue;
        }

        d = (box->top_clip - src[v1].y) / (src[v2].y - src[v1].y);

        dst[c].y = box->top_clip;
        dst[c].x = src[v1].x + d * (src[v2].x - src[v1].x);
        c++;
    }
    return(c);
}
