//***************************************************************************
//
// File         : input.cpp
//
// Description  : DOS/DJGPP Protected mode keyboard/mouse routines...
//                Written by Lars Andersson (lars_and@algonet.se), 1998
//
//***************************************************************************

#define INPUT

#include <stdio.h>
#include <dpmi.h>
#include <dos.h>
#include <go32.h>
#include <pc.h>

#include "input.h"

#define KEY_INT 0x9
#define MOUSE_INT 0x33

_go32_dpmi_seginfo  old_keyhandler, new_keyhandler;

char transtab[128] = {0, 0, 49, 50, 51, 52, 53, 54, 55, 56, 57, 48, 43, 0, 8,
                0, 81, 87, 69, 82, 84, 89, 85, 73, 79, 80, 0, 0, 13, 0,
                65, 83, 68, 70, 71, 72, 74, 75, 76, 58, 39, 0, 0, 0,
                90, 88, 67, 86, 66, 78, 77, 44, 46, 45, 0, 0, 0, 32,
                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 55, 56, 57, 45,
                52, 53, 54, 43, 49, 50, 51, 0, 0};


int         mousex, mousey, mousedx, mousedy, mousebuttons;
volatile    char keys[128];
volatile    char hit, lastcode, special;
char        keyboard_installed = FALSE;

//--- The keyboard interrupt handler ----------------------------------------

void KeyHandler(void)
{
    unsigned char code = (char)inp(0x60);

       if (code < 128) {
            keys[code] = 1;
            hit = 1;
            if (special) {
                special = FALSE;
                lastcode = 0;
            } else {
                lastcode = code;
            }
        }
        else {
            if (special) special = FALSE;
            if (code == 224) special = TRUE;
            else keys[code-128] = 0;
        }

    code = (char)inp(0x61);
    outp(0x61, code | 0x80);
    outp(0x20, 0x20);
}

//--- Install our new protected mode Keyboard ISR ---------------------------

void Init_Keyboard(void)
{
    if (!keyboard_installed) {

        for (int i = 0; i < 128; i++) keys[i] = FALSE;
        keyboard_installed = TRUE; hit = special = FALSE;

        new_keyhandler.pm_offset   = (int)KeyHandler;
        new_keyhandler.pm_selector = _go32_my_cs();
        _go32_dpmi_get_protected_mode_interrupt_vector(0x9, &old_keyhandler);
        _go32_dpmi_allocate_iret_wrapper(&new_keyhandler);
        _go32_dpmi_set_protected_mode_interrupt_vector(0x9, &new_keyhandler);
    }
}

//--- Remove Keyboard ISR ---------------------------------------------------

void Close_Keyboard(void)
{
    if (keyboard_installed) {
        _go32_dpmi_set_protected_mode_interrupt_vector(0x9, &old_keyhandler);
        keyboard_installed = FALSE;
    }
}

//--- Initialize mouse ------------------------------------------------------

int Init_Mouse(void)
{
    union REGS regs;

    regs.x.ax = 0;
    int86(MOUSE_INT, &regs, &regs);
    return((int)regs.x.ax);
}

//--- Close Mouse -----------------------------------------------------------

void Close_Mouse(void)
{
}

//--- Update Input ----------------------------------------------------------

void Update_Input(void)
{
    union REGS regs;

    regs.x.ax = 3;
    int86(MOUSE_INT, &regs, &regs);
    mousebuttons = regs.w.bx;
    mousex = regs.x.cx;
    mousey = regs.x.dx;

    regs.x.ax = 0xb;
    int86(MOUSE_INT, &regs, &regs);
    mousedx = (int)((short)(regs.x.cx));
    mousedy = (int)((short)(regs.x.dx));
}

//--- Returns TRUE if a key has been hit since last call --------------------

char KeyHit(void)
{
    if (hit) {
        hit=FALSE;
        return TRUE;
    } else return FALSE;
}

//--- Returns TRUE if key has been hit and waits until it's released --------

char TestKey(int key)
{
    if (keys[key]) {
        while(keys[key]);
        hit=FALSE;
        return TRUE;
    } else return FALSE;
}

//--- Wait for key and return ascii code ------------------------------------

char GetChar(void)
{
    char    ch;

    while (!hit);
    hit = 0;
    ch = transtab[lastcode];
    if (ch != 0) keys[lastcode] = 0;
    return(ch);
}

//--- Set mouse sensitivity -------------------------------------------------

void Set_MouseSensitivity(int s, int d)
{
    union REGS regs;

    regs.x.ax = 0x1a;
    regs.x.bx = s;
    regs.x.cx = s;
    regs.x.dx = d;

    int86(MOUSE_INT, &regs, &regs);
}
