//****************************************************************************
//
// File         : engine.h
//
// Description  : Main engine structures and definitions.
//                Written by Lars Andersson (lars_and@algonet.se), 1998
//
//****************************************************************************

#ifndef ENGINE_H
#define ENGINE_H

#ifndef VERTEX_H
    #include "vector.h"
#endif

#ifndef VERTEX_H
    #include "camera.h"
#endif


//--- Misc constants --------------------------------------------------------

#define MAX_YRESOLUTION 768
#define PORTALS_PER_SECTOR 8
#define VERTICES_PER_POLYGON 8

//--- Main engine flags -----------------------------------------------------

#define ENGINE_TMAPPING 1

//--- Polygon flags ---------------------------------------------------------

#define POLY_PORTAL 1
#define POLY_CROSSABLE 2
#define POLY_TEXTUREMAPPED 4
#define POLY_GOURAUD 8
#define POLY_CLIPPED 16

//--- Structure definitions -------------------------------------------------

typedef struct {
    int       sx, ex;
} HLine;

typedef struct {
    float       top_clip, bottom_clip, right_clip, left_clip;
    HLine       hclip[MAX_YRESOLUTION];
} Clip;

#ifndef SCREEN_H
    #include "screen.h"
#endif

typedef struct {
    void        *sector_p;
    short       nr_vertices, pad;
    Vertice_2DI xy[VERTICES_PER_POLYGON * 2];
} Portal;

typedef struct {
    int         timestamp;
    Vector_3D   p, m, n;
    Vector_3D   a, b, c;
    char        *texture_p;
} TPlane;

typedef struct {
    int         attribute;
    short       flags;
    short       nr_vertices;
    short       pindex[VERTICES_PER_POLYGON];
    Vector_3D   normal;
    TPlane      *tplane_p;
    float       d;
} Polygon;

typedef struct {
    short       id, nr_vertices, nr_polygons;
    short       flags;
    Vertice_3D  *vertices_p;
    Polygon     *polygons_p;
    TPlane      *tplanes_p;
    Portal      portals[PORTALS_PER_SECTOR];
    Clip        clip;
} Sector;

typedef struct {
    Vertice_3D  *vertices_p;
    Polygon     *polygons_p;
    Sector      *sectors_p;
    short       id, nr_vertices, nr_polygons, nr_sectors;
    int         flags;
} Object;

//--- Global engine data structure ------------------------------------------

typedef struct {
    int         flags, ticks, total_ticks, framecounter;
    int         nr_objects, nr_sectors, nr_polygons, nr_portals;
    int         nr_textures, nr_transformed;
    Object      *objects_p;
    Sector      *csector_p;
    char        *textures_p;
    char        *palette_p;
    Vertice_3D  *pVertices;
    Vertice_3D  *transformed_p;
    Vertice_2DD *projected_p;
    ScrMode     screen;
    Camera      camera;
} Engine;

//--- Function declarations -------------------------------------------------

int     Init_Engine(void);
void    Close_Engine(void);
void    Render_Frame(void);
void    Set_RenderMode(int mode);

void    Update_CSector(void);
void    Draw_Sector(Sector *sector);
void    Transform_Sector(Sector *sector);
void    Project_Sector(Sector *sector);
void    Setup_TPlane(TPlane *tp);

void    Reset_Timer(void);

#endif
