//****************************************************************************
//
// File         : fixed.h
//
// Description  : Fixed point defs and macros.
//                Written by Lars Andersson (lars_and@algonet.se), 1998
//
//****************************************************************************

#ifndef FIXED_H
#define FIXED_H

#define FIXED_POINT 20
#define FIXED_ONE (1<<FIXED_POINT)
#define FIXED_MASK (0xffffffff >> (32-FIXED_POINT))

#define F2I_Round(x) (((x) + (FIXED_ONE>>1)) >> FIXED_POINT)
#define F2I_Ceil(x) (((x) + FIXED_ONE - 1) >> FIXED_POINT)
#define Fixed_Ceil(x) (((x) + FIXED_ONE - 1) & ~FIXED_MASK)

int Fixed_Mul64(int x, int y);
int Fixed_Div64(int x, int y);

//--- Watcom C/C++ inline asm -----------------------------------------------

#ifdef COMP_WCC

#pragma aux Fixed_Mul64=\
               "imul  ebx           "\
               "shrd  eax, edx, 20  "\
               parm [eax] [ebx]      \
               modify [eax edx]      \
               value [eax];

#pragma aux Fixed_Div64=\
               "cdq                 "\
               "shld  edx, eax, 20  "\
               "sal   eax, 20       "\
               "idiv  ebx           "\
               parm [eax] [ebx]      \
               modify [eax edx]      \
               value [eax];
#endif

//--- DJGPP/GCC inline asm --------------------------------------------------

#if defined(COMP_DJGPP) || defined(COMP_GCC)

inline int Fixed_Mul64(int x, int y)
{

    int tmp;

    asm (" imull   %%ebx                  \n\t"
         " shrdl   $20, %%edx, %%eax          "
         : "=a" (tmp)
         : "a" (x), "b" (y)       /* gcc chooses reg for y. */
         : "%eax", "%edx");

//    tmp = (int)((((long long)x) * y) >> FIXED_POINT);

    return(tmp);
}

inline int Fixed_Div64(int x, int y)
{
    int tmp;

  asm ("cdq                         \n\t"
       "shldl   $20, %%eax, %%edx   \n\t"
       "shll    $20, %%eax          \n\t"
       "idivl   %%ebx"
       : "=a" (tmp)
       : "a" (x), "b" (y)       /* gcc chooses reg for y. */
       : "%eax", "%edx");

//    tmp = (int)((((long long)(x)) << FIXED_POINT) / y);

    return(tmp);
}
#endif
#endif
