//****************************************************************************
//
// File        : flat.cpp
//
// Description : Solid polygon scan conversion code...
//               Written by Lars Andersson (lars_and@algonet.se), 1997
//
//****************************************************************************

#include "main.h"
#include "poly.h"
#include "fpu.h"
#include "fixed.h"

/*---------------------------------------------------------------------------
 Draw_FlatPolygon() - Draws a solid colored polygon with edge clipping
---------------------------------------------------------------------------*/

void Draw_FlatPolygon(Vertice_2DI *coord, int points, int color)
{
    char           *scroffs, *scrpos;
    int            i, y, ydif, miny, maxy, slen;
    int            lx, rx, ldx, rdx, lcount, rcount;
    int            top_index, bottom_index, ledge, redge;

    // Setup color dword

    color |= (color << 8);
    color |= (color << 16);

    // Find top and bottom points

    miny = maxy = coord[0].y;
    top_index = bottom_index = 0;

    for (i = 1; i < points; i++) {
       if (coord[i].y < miny) {
          miny = coord[i].y;
          top_index = i;
       }
       if (coord[i].y > maxy) {
          maxy = coord[i].y;
          bottom_index = i;
       }
    }

    if (maxy == miny) return;
    miny = F2I_Ceil(miny);

    scroffs = (char *)(engine.screen.buffer + engine.screen.ymultab[miny]);
 
    ledge = redge = top_index;
    lcount = rcount = 0;

    // Loop forever

    while (1) {

        // New left edge?

        if (--lcount < 0) {

            if (ledge == bottom_index) return;
            i = ledge + 1;
            if (i == points) i = 0;
            lcount = F2I_Ceil(coord[i].y - FIXED_ONE) - F2I_Ceil(coord[ledge].y);

            while (lcount < 0) {
                if (i == bottom_index) return;
                ledge = i;
                i++;
                if (i == points) i = 0;
                lcount = F2I_Ceil(coord[i].y - FIXED_ONE) - F2I_Ceil(coord[ledge].y);
            }

            // Calculate edge gradients

            y = (coord[ledge].y + (FIXED_ONE - 1)) & ~FIXED_MASK;
            ydif = coord[i].y - coord[ledge].y;

            if (ydif < (FIXED_ONE / 2)) ydif = (FIXED_ONE / 2);

            ldx = Fixed_Div64(coord[i].x - coord[ledge].x, ydif);
            lx = coord[ledge].x;
            lx += Fixed_Mul64(y - coord[ledge].y, ldx);

            ledge = i;
        }

        // New right edge?

        if (--rcount < 0) {

            if (redge == bottom_index) return;
            i = redge-1;
            if (i < 0) i = points - 1;

            rcount = F2I_Ceil(coord[i].y - FIXED_ONE) - F2I_Ceil(coord[redge].y);

            while (rcount < 0) {
                if (i == bottom_index) return;
                redge = i;
                i--;
                if (i < 0) i = points - 1;
                rcount = F2I_Ceil(coord[i].y - FIXED_ONE) - F2I_Ceil(coord[redge].y);
            }

            // Calculate edge gradients

            y = (coord[redge].y + (FIXED_ONE - 1)) & ~FIXED_MASK;
            ydif = coord[i].y - coord[redge].y;
            if (ydif < (FIXED_ONE / 2)) ydif = (FIXED_ONE / 2);

            rdx = Fixed_Div64(coord[i].x - coord[redge].x, ydif);
            rx = coord[redge].x;
            rx += Fixed_Mul64(y - coord[redge].y, rdx);

            redge = i;
        }

        int slx = lx;
        int srx = rx;

        if (slx < engine.screen.cclip->hclip[miny].sx)
            slx = engine.screen.cclip->hclip[miny].sx;
        if (srx > engine.screen.cclip->hclip[miny].ex)
            srx = engine.screen.cclip->hclip[miny].ex;

        slx = F2I_Ceil(slx);
        srx = F2I_Ceil(srx - FIXED_ONE);

        slen = srx - slx + 1;

        if (slen > 0) {
            scrpos = (char *)scroffs + slx;
            Hline(scrpos, slen, color);
        }

        lx += ldx;
        rx += rdx;
        miny++;
        scroffs += engine.screen.bytes_per_row;
    }
}

