//****************************************************************************
//
// File         : input.h
//
// Description  : Input code include file.
//                Written by Lars Andersson (lars_and@algonet.se), 1998
//
//****************************************************************************

#ifndef INPUT_H
#define INPUT_H

#define LEFT_MBUTTON 1
#define RIGHT_MBUTTON 2
#define KEY(x) keys[x]

#ifndef INPUT
   extern volatile char keys[];
   extern int mousedx, mousedy, mousebuttons;
#endif

#ifndef TRUE
   #define TRUE 1
#endif

#ifndef FALSE
   #define FALSE 0
#endif

//--- Keyboard controller codes ---------------------------------------------

#if !defined(X11)

#define A_KEY 30
#define B_KEY 48
#define C_KEY 46
#define D_KEY 32
#define E_KEY 18
#define F_KEY 33
#define G_KEY 34
#define H_KEY 35
#define I_KEY 23
#define J_KEY 36
#define L_KEY 38
#define M_KEY 50
#define N_KEY 49
#define O_KEY 24
#define P_KEY 25
#define R_KEY 19
#define T_KEY 20
#define X_KEY 45
#define Y_KEY 21
#define Z_KEY 44
#define M_KEY 50
#define Q_KEY 16

#define HOME_KEY 71
#define END_KEY 79
#define INS_KEY 82
#define DEL_KEY 83

#define ESC_KEY 1
#define ENTER_KEY 28

#define PLUS_KEY 78
#define MINUS_KEY 74

#define SPACE_KEY 57
#define CTRL_KEY 29
#define LALT_KEY 56
#define LSHIFT_KEY 42
#define RSHIFT_KEY 54
#define CAPSLOCK_KEY 58
#define TAB_KEY 15

#define F1_KEY 59
#define F2_KEY 60
#define F3_KEY 61
#define F4_KEY 62
#define F5_KEY 63
#define F6_KEY 64
#define F7_KEY 65
#define F8_KEY 66
#define F9_KEY 67
#define F10_KEY 68
#define F11_KEY 87
#define F12_KEY 88

#if defined(SVGALIB)

#define UP_KEY 103
#define DOWN_KEY 108
#define LEFT_KEY 105
#define RIGHT_KEY 106 

#define PGUP_KEY 104
#define PGDN_KEY 109

#else

#define UP_KEY 72
#define DOWN_KEY 80
#define LEFT_KEY 75
#define RIGHT_KEY 77 

#define PGUP_KEY 73
#define PGDN_KEY 81

#endif

#else

#define BACKSPACE_KEY            0x08  /* back space, back char */
#define TAB_KEY                  0x09
#define RETURN_KEY      	     0x0D  /* Return, enter */
#define PAUSE_KEY                0x13  /* Pause, hold */
#define SCRLOCK_KEY              0x14
#define SYSREQ_KEY               0x15
#define ESC_KEY                  0x1B
#define DELETE_KEY               0xFF  /* Delete, rubout */

#define HOME_KEY                 0x50
#define LEFT_KEY                 0x51  /* Move left, left arrow */
#define UP_KEY                   0x52  /* Move up, up arrow */
#define RIGHT_KEY                0x53  /* Move right, right arrow */
#define DOWN_KEY                 0x54  /* Move down, down arrow */
#define PGUP_KEY                 0x55
#define PGDN_KEY                 0x56
#define END_KEY                  0x57  /* EOL */

#define PRINT_KEY                0x61
#define INSERT_KEY		         0x63
#define HELP_KEY                 0x6A  /* Help */
#define BREAK_KEY                0x6B
#define NUMLOCK_KEY              0x7F

#define F1_KEY                   0xBE
#define F2_KEY                   0xBF
#define F3_KEY                   0xC0
#define F4_KEY                   0xC1
#define F5_KEY                   0xC2
#define F6_KEY                   0xC3
#define F7_KEY                   0xC4
#define F8_KEY                   0xC5
#define F9_KEY                   0xC6
#define F10_KEY                  0xC7
#define F11_KEY                  0xC8
#define F12_KEY                  0xC9

#define LSHIFT_KEY               0xE1  /* Left shift */
#define RSHIFT_KEY               0xE2  /* Right shift */
#define LCONTROL_KEY             0xE3  /* Left control */
#define RCONTROL_KEY             0xE4  /* Right control */
#define CAPSLOCK_KEY             0xE5  /* Caps lock */

#define LMETA_KEY                0xE7  /* Left meta */
#define RMETA_KEY                0xE8  /* Right meta */
#define LALT_KEY                 0xE9  /* Left alt */
#define RALT_KEY                 0xEA  /* Right alt */

#define SPACE_KEY                0x020
#define PLUS_KEY	             0x02b	
#define COMMA_KEY                0x02c
#define MINUS_KEY                0x02d
#define PERIOD_KEY               0x02e
#define SLASH_KEY                0x02f
#define ZERO_KEY                 0x030
#define ONE_KEY                  0x031
#define TWO_KEY                  0x032
#define THREE_KEY                0x033
#define FOUR_KEY                 0x034
#define FIVE_KEY                 0x035
#define SIX_KEY                  0x036
#define SEVEN_KEY                0x037
#define EIGHT_KEY                0x038
#define NINE_KEY                 0x039
#define COLON_KEY                0x03a
#define SEMICOLON_KEY            0x03b
#define LESS_KEY                 0x03c
#define EQUAL_KEY                0x03d
#define GREATER_KEY              0x03e

#define A_KEY                    0x061
#define B_KEY                    0x062
#define C_KEY                    0x063
#define D_KEY                    0x064
#define E_KEY                    0x065
#define F_KEY                    0x066
#define G_KEY                    0x067
#define H_KEY                    0x068
#define I_KEY                    0x069
#define J_KEY                    0x06a
#define K_KEY                    0x06b
#define L_KEY                    0x06c
#define M_KEY                    0x06d
#define N_KEY                    0x06e
#define O_KEY                    0x06f
#define P_KEY                    0x070
#define Q_KEY                    0x071
#define R_KEY                    0x072
#define S_KEY                    0x073
#define T_KEY                    0x074
#define U_KEY                    0x075
#define V_KEY                    0x076
#define W_KEY                    0x077
#define X_KEY                    0x078
#define Y_KEY                    0x079
#define Z_KEY                    0x07a

#define BACKSLASH_KEY            0x05c
#define UNDERSCORE_KEY           0x05f
#define VERS_A_KEY               0x061
#define VERS_Z_KEY               0x07a

#endif


//--- Function defs ---------------------------------------------------------

void    Init_Keyboard(void);
void    Close_Keyboard(void);
void	Update_Input(void);
char    KeyHit(void);
char    TestKey(int key);
char    GetChar(void);

int     Init_Mouse(void);
void    Close_Mouse(void);
void    Set_MouseSensitivity(int s);

#endif
