//****************************************************************************
//
// File         : tweaked.h
//
// Description  : DOS Tweaked mode code header file.
//                Written by Lars Andersson (lars_and@algonet.se), 1997
//
//****************************************************************************

#ifndef TWEAKED_H
#define TWEAKED_H

#define MODE_320x240 0
#define MODE_320x400 1
#define MODE_360x270 2
#define MODE_360x360 3
#define MODE_360x400 4

#define ATTRCON_ADDR    0x3C0
#define MISC_ADDR       0x3C2
#define VGAENABLE_ADDR  0x3C3
#define SEQ_ADDR        0x3C4
#define GRACON_ADDR     0x3CE
#define CRTC_ADDR       0x3D4
#define STATUS_ADDR     0x3DA

// Turn on structure packing

#ifdef COMP_WCC
    #pragma pack (push);
    #pragma pack (1);         // Watcom C++ : No structure member alignement
#endif

#if defined(COMP_DJGPP) || defined(COMP_GCC)
    #pragma pack(1)           // DJGPP / GCC : No structure member alignement
#endif

typedef struct {
   unsigned short port;
   unsigned char index;
   unsigned char value;
} Register;

// Turn off structure packing

#ifdef COMP_WCC
    #pragma pack (pop);
#endif

#if defined(COMP_DJGPP) || defined(COMP_GCC)
    #pragma pack()
#endif


Register Mode_320x240regs[] = {
   { 0x3c2, 0x0, 0xe3},
   { 0x3d4, 0x0, 0x5f},
   { 0x3d4, 0x1, 0x4f},
   { 0x3d4, 0x2, 0x50},
   { 0x3d4, 0x3, 0x82},
   { 0x3d4, 0x4, 0x54},
   { 0x3d4, 0x5, 0x80},
   { 0x3d4, 0x6, 0xd},
   { 0x3d4, 0x7, 0x3e},
   { 0x3d4, 0x8, 0x0},
   { 0x3d4, 0x9, 0x41},
   { 0x3d4, 0x10, 0xea},
   { 0x3d4, 0x11, 0xac},
   { 0x3d4, 0x12, 0xdf},
   { 0x3d4, 0x13, 0x28},
   { 0x3d4, 0x14, 0x0},
   { 0x3d4, 0x15, 0xe7},
   { 0x3d4, 0x16, 0x6},
   { 0x3d4, 0x17, 0xe3},
   { 0x3c4, 0x1, 0x1},
   { 0x3c4, 0x4, 0x6},
   { 0x3ce, 0x5, 0x40},
   { 0x3ce, 0x6, 0x5},
   { 0x3c0, 0x10, 0x41},
   { 0x3c0, 0x13, 0x0}
};

Register Mode_320x400regs[] = {
   { 0x3c2, 0x0, 0x63},
   { 0x3d4, 0x0, 0x5f},
   { 0x3d4, 0x1, 0x4f},
   { 0x3d4, 0x2, 0x50},
   { 0x3d4, 0x3, 0x82},
   { 0x3d4, 0x4, 0x54},
   { 0x3d4, 0x5, 0x80},
   { 0x3d4, 0x6, 0xbf},
   { 0x3d4, 0x7, 0x1f},
   { 0x3d4, 0x8, 0x0},
   { 0x3d4, 0x9, 0x40},
   { 0x3d4, 0x10, 0x9c},
   { 0x3d4, 0x11, 0x8e},
   { 0x3d4, 0x12, 0x8f},
   { 0x3d4, 0x13, 0x28},
   { 0x3d4, 0x14, 0x0},
   { 0x3d4, 0x15, 0x96},
   { 0x3d4, 0x16, 0xb9},
   { 0x3d4, 0x17, 0xe3},
   { 0x3c4, 0x1, 0x1},
   { 0x3c4, 0x4, 0x6},
   { 0x3ce, 0x5, 0x40},
   { 0x3ce, 0x6, 0x5},
   { 0x3c0, 0x10, 0x41},
   { 0x3c0, 0x13, 0x0}
};

Register Mode_360x270regs[] = {
   { 0x3c2, 0x0, 0xe7},
   { 0x3d4, 0x0, 0x6b},
   { 0x3d4, 0x1, 0x59},
   { 0x3d4, 0x2, 0x5a},
   { 0x3d4, 0x3, 0x8e},
   { 0x3d4, 0x4, 0x5e},
   { 0x3d4, 0x5, 0x8a},
   { 0x3d4, 0x6, 0x30},
   { 0x3d4, 0x7, 0xf0},
   { 0x3d4, 0x8, 0x0},
   { 0x3d4, 0x9, 0x61},
   { 0x3d4, 0x10, 0x20},
   { 0x3d4, 0x11, 0xa9},
   { 0x3d4, 0x12, 0x1b},
   { 0x3d4, 0x13, 0x2d},
   { 0x3d4, 0x14, 0x0},
   { 0x3d4, 0x15, 0x1f},
   { 0x3d4, 0x16, 0x2f},
   { 0x3d4, 0x17, 0xe3},
   { 0x3c4, 0x1, 0x1},
   { 0x3c4, 0x4, 0x6},
   { 0x3ce, 0x5, 0x40},
   { 0x3ce, 0x6, 0x5},
   { 0x3c0, 0x10, 0x41},
   { 0x3c0, 0x13, 0x0}
};

Register Mode_360x360regs[] = {
   { 0x3c2, 0x0, 0x67},
   { 0x3d4, 0x0, 0x6b},
   { 0x3d4, 0x1, 0x59},
   { 0x3d4, 0x2, 0x5a},
   { 0x3d4, 0x3, 0x8e},
   { 0x3d4, 0x4, 0x5e},
   { 0x3d4, 0x5, 0x8a},
   { 0x3d4, 0x6, 0xbf},
   { 0x3d4, 0x7, 0x1f},
   { 0x3d4, 0x8, 0x0},
   { 0x3d4, 0x9, 0x40},
   { 0x3d4, 0x10, 0x88},
   { 0x3d4, 0x11, 0x85},
   { 0x3d4, 0x12, 0x67},
   { 0x3d4, 0x13, 0x2d},
   { 0x3d4, 0x14, 0x0},
   { 0x3d4, 0x15, 0x6d},
   { 0x3d4, 0x16, 0xba},
   { 0x3d4, 0x17, 0xe3},
   { 0x3c4, 0x1, 0x1},
   { 0x3c4, 0x4, 0x6},
   { 0x3ce, 0x5, 0x40},
   { 0x3ce, 0x6, 0x5},
   { 0x3c0, 0x10, 0x41},
   { 0x3c0, 0x13, 0x0}
};

Register Mode_360x400regs[] = {
   { 0x3c2, 0x0, 0x67},
   { 0x3d4, 0x0, 0x6b},
   { 0x3d4, 0x1, 0x59},
   { 0x3d4, 0x2, 0x5a},
   { 0x3d4, 0x3, 0x8e},
   { 0x3d4, 0x4, 0x5e},
   { 0x3d4, 0x5, 0x8a},
   { 0x3d4, 0x6, 0xbf},
   { 0x3d4, 0x7, 0x1f},
   { 0x3d4, 0x8, 0x0},
   { 0x3d4, 0x9, 0x40},
   { 0x3d4, 0x10, 0x9c},
   { 0x3d4, 0x11, 0x8e},
   { 0x3d4, 0x12, 0x8f},
   { 0x3d4, 0x13, 0x2d},
   { 0x3d4, 0x14, 0x0},
   { 0x3d4, 0x15, 0x96},
   { 0x3d4, 0x16, 0xb9},
   { 0x3d4, 0x17, 0xe3},
   { 0x3c4, 0x1, 0x1},
   { 0x3c4, 0x4, 0x6},
   { 0x3ce, 0x5, 0x40},
   { 0x3ce, 0x6, 0x5},
   { 0x3c0, 0x10, 0x41},
   { 0x3c0, 0x13, 0x0}
};

void    OutReg(Register r);

#endif
