//****************************************************************************
//
// File         : vesa2.h
//
// Description  : VESA 2.0 header file. Based on VGOVBE20.H by
//                TimJ (tim@legend.co.uk) of Vertigo.
//                Modified by Lars Andersson (lars_and@algonet.se), 1997
//
//****************************************************************************

#ifndef VESA2_H
#define VESA2_H

#define VBE2 0x32454256             // 'VBE2'
#define VESA 0x41534556             // 'VESA'


#if defined(COMP_WCC)
    #define FLAT_PTR(x) ((((long)x>>12) & 0xffff0) + ((long)x & 0xffff))
#endif

#if defined(COMP_DJGPP)
    #define FLAT_PTR(x) (__djgpp_conventional_base + (((long)x>>12) & 0xffff0) + ((long)x & 0xffff))
#endif


// Turn on structure packing

#ifdef COMP_WCC
    #pragma pack (push);
    #pragma pack (1);         // Watcom C++ : No structure member alignement
#endif

#if defined(COMP_DJGPP) || defined(COMP_GCC)
    #pragma pack(1)           // DJGPP / GCC : No structure member alignement
#endif


// Register structure used for DPMI functions

typedef struct {
   long        edi,esi,ebp,esp,ebx,edx,ecx,eax;
   short       flags,es,ds,fs,gs,ip,cs,sp,ss;
   long        spare_data[20];
} RMRegs;

// VESA infomation block

typedef struct
{
  long        VbeSignature;         // 'VESA' 4 byte signature
  short 	  VbeVersion;           // VBE version number
  char		 *OemStringPtr;         // Pointer to OEM string
  unsigned long   Capabilities;     // Capabilities of video card
  unsigned short *VideoModePtr;		// Pointer to supported modes
  short   	  TotalMemory;          // Number of 64kb memory blocks

  short       OemSoftwareRev;       // OEM Software revision number
  char       *OemVendorNamePtr;     // Pointer to Vendor Name string
  char		 *OemProductNamePtr;	// Pointer to Product Name string
  char       *OemProductRevPtr;     // Pointer to Product Revision str
  char  	  reserved[222];       	// Pad to 256 byte block size
  char        OemDATA[256];         // Scratch pad for OEM data

} VBEInfo;

#define VBE_CAPS_8BITDAC    (1<<0)      // DAC can be 8 bit
#define VBE_CAPS_NONVGA     (1<<1)      // Non-vga compatible controller
#define	VBE_CAPS_BLANKRAMDAC 	(1<<2)	// Programmed DAC with blank bit

// VESA mode infomation block

typedef struct
{
  // Mandatory information for all VBE revisions:

  unsigned short  ModeAttributes;        // mode attributes
  unsigned char	  WinAAttributes;        // window A attributes
  unsigned char   WinBAttributes;        // window B attributes
  unsigned short  WinGranularity;        // window granularity
  unsigned short  WinSize;               // window size
  unsigned short  WinASegment;           // window A start segment
  unsigned short  WinBSegment;           // window B start segment
  unsigned long	  WinFuncPtr;            // pointer to window function
  unsigned short  BytesPerScanLine;      // bytes per scan line

  // Mandatory information for VBE 1.2 and above:

  unsigned short  XResolution;           // horizontal resolution in pixels or chars
  unsigned short  YResolution;           // vertical resolution in pixels or chars
  unsigned char   XCharSize;             // character cell width in pixels
  unsigned char	  YCharSize;             // character cell height in pixels
  unsigned char	  NumberOfPlanes;        // number of memory planes
  unsigned char	  BitsPerPixel;          // bits per pixel
  unsigned char	  NumberOfBanks;         // number of banks
  unsigned char	  MemoryModel;           // memory model type
  unsigned char	  BankSize;              // bank size in KB
  unsigned char	  NumberOfImagePages;    // number of images
  unsigned char	  _Reserved;             // reserved for page function

  // Direct Color fields (required for direct/6 and YUV/7 memory models)

  unsigned char	  RedMaskSize;           // size of direct color red mask in bits
  unsigned char	  RedFieldPosition;      // bit position of lsb of red mask
  unsigned char	  GreenMaskSize;         // size of direct color green mask in bits
  unsigned char	  GreenFieldPosition;    // bit position of lsb of green mask
  unsigned char	  BlueMaskSize;          // size of direct color blue mask in bits
  unsigned char	  BlueFieldPosition;     // bit position of lsb of blue mask
  unsigned char	  RsvdMaskSize;          // size of direct color reserved mask in bits
  unsigned char	  RsvdFieldPosition;     // bit position of lsb of reserved mask
  unsigned char	  DirectColorModeInfo;   // direct color mode attributes

  // Mandatory information for VBE 2.0 and above:

  unsigned long	  PhysBasePtr;           // physical address for flat frame buffer
  unsigned long	  OffScreenMemOffset;    // pointer to start of off screen memory
  unsigned short  OffScreenMemSize;      // amount of off screen memory in 1k units
  char            _ReservedChunk[206];

} VBEModeInfo;


// Turn off structure packing

#ifdef COMP_WCC
    #pragma pack (pop);
#endif

#if defined(COMP_DJGPP) || defined(COMP_GCC)
    #pragma pack()
#endif


// Flags to combine with mode number

#define	VBE_MODEFLAG_NOCLEAR	0x8000	// Dont clear screen on mode set
#define VBE_MODEFLAG_LFB    0x4000      // Request LFB

// Mode attributes flags

#define VBEMODE_CAPS_AVAILABLE	(1<<0)	// Mode is available
#define	VBEMODE_CAPS_TTYOUTPUT	(1<<2)	// TTY output is available
#define	VBEMODE_CAPS_COLORMODE	(1<<3)	// Color video mode
#define	VBEMODE_CAPS_GRAPHMODE	(1<<4)	// Graphics mode
#define	VBEMODE_CAPS_NONVGA	(1<<5)	// Non-VGA mode
#define	VBEMODE_CAPS_NONBANKED	(1<<6)	// Banked frame buffer not supported
#define	VBEMODE_CAPS_LINEAR	(1<<7)	// Linear frame buffer is supported

// Mode memory models

#define	VBEMODE_MODEL_TEXT	(0)	// Text mode
#define	VBEMODE_MODEL_CGA	(1)	// CGA mode
#define	VBEMODE_MODEL_HERCULES	(2)	// Hercules graphics mode
#define	VBEMODE_MODEL_PLANAR	(3)	// VGA planar style mode
#define	VBEMODE_MODEL_PACKED	(4)	// Packed pixel mode
#define	VBEMODE_MODEL_MODEX	(5)	// ModeX (unchained 4, 256 color)
#define	VBEMODE_MODEL_RGB	(6)	// Direct color RGB
#define	VBEMODE_MODEL_YUV	(7)	// Direct color YUV

#endif

