//****************************************************************************
//
// File         : video.h
//
// Description  : Video mode structures and functions defs.
//                Written by Lars Andersson (lars_and@algonet.se), 1998
//
//****************************************************************************

#ifndef VIDEO_H
#define VIDEO_H

//--- Video pragmas ---------------------------------------------------------

#if defined(COMP_WCC)

void Dword_Fill(char *dest, int filler, int dwords);
#pragma aux Dword_Fill = \
                "rep    stosd" \
                parm [edi] [eax] [ecx];

void Dword_Copy(char *dest, char *source, int dwords);
#pragma aux Dword_Copy = \
                "cld         " \
                "rep    movsd" \
                parm [edi] [esi] [ecx];

void Dword_CopyClear(char *dest, char *source, int filler, int dwords);
#pragma aux Dword_CopyClear = \
            "CCLoop:                "\
                "mov    ebx, [esi]  "\
                "mov    [edi], ebx  "\
                "add    edi, 4      "\
                "mov    [esi], eax  "\
                "add    esi, 4      "\
                "dec    ecx         "\
                "jnz    CCLoop      "\
                modify [ebx]         \
                parm [edi] [esi] [eax] [ecx];

void WaitRetrace_ASM(void);
#pragma aux WaitRetrace_ASM=\
                    "mov    dx, 03dah"\
				    "start: in al, dx"\
				    "test   al, 8"\
				    "jnz    start"\
				    "mov    dx, 03dah"\
				    "end:   in al, dx"\
				    "test   al, 8"\
				    "jz     end"\
                     modify [al dx];

void SetPalette_ASM(char *palette);
#pragma aux SetPalette_ASM=\
                    "cld"\
                    "mov   edx, 3c8h"\
                    "xor   eax, eax"\
                    "out   dx, al"\
                    "inc   edx"\
                    "mov   ecx, 768"\
                    "rep   outsb"\
                    parm[esi]\
                    modify[eax ecx edx esi];
#endif


#if defined(COMP_DJGPP) || defined(COMP_GCC)

#define Dword_Fill(dest, filler, dwords) \
__asm__(" cld       \n\t"\
        " rep       \n\t"\
        " stosl         "\
        : : "D" (dest), "a" (filler), "c" (dwords)  \
        : "%ecx", "%edi")

#define Dword_Copy(dest, source, dwords) \
__asm__(" cld       \n\t"\
        " rep       \n\t"\
        " movsl         "\
        : : "D" (dest), "S" (source), "c" (dwords) \
        : "%ecx", "%edi", "%esi")

#define Dword_CopyClear(dest, source, filler, dwords) \
__asm__(" 1:                    \n\t"\
        " movl  (%%esi), %%ebx  \n\t"\
        " movl  %%ebx, (%%edi)  \n\t"\
        " addl  $4, %%edi       \n\t"\
        " movl  %%eax, (%%esi)  \n\t"\
        " addl  $4, %%esi       \n\t"\
        " decl  %%ecx           \n\t"\
        " jnz   1b                  "\
        : : "D" (dest), "S" (source), "a" (filler), "c" (dwords) \
        : "%eax", "%ebx", "%ecx", "%edi", "%esi")

#endif

//--- Function defs ---------------------------------------------------------

int     Init_Video(void);
void    Close_Video();

int     Set_VideoMode(int mode);
void    Close_VideoMode();

void    Wait_Retrace(void);
void    Set_Palette(char *palette);
int     Set_ScanWidth(long width);
int     Set_DisplayStart(long x ,long y);

void    Tweaked_ClearBlock(int x, int y, int xsize, int ysize, int color);
void    Tweaked_CopyBlock(int x, int y, int xsize, int ysize, char *source);

void    Copy_Buffer(void);
void    Clear_Buffer(void);
void    Copy_XVGA(void);
void    Clear_VGA(void);
void    Clear_XVGA(void);

#endif
