//****************************************************************************
//
// File         : matrix.cpp
//
// Description  : 4x4 Matrix routines. By concatenation I mean multiply and
//                copy to first operand. By multiplication I mean multiply
//                and copy to new matrix.
//
//****************************************************************************

#include <stdio.h>
#include <string.h>

#include "main.h"
#include "matrix.h"

//--- Create identity matrix ------------------------------------------------

void Mat_Identity_4x4(Matrix_4x4 m)
{
    memset(m,NULL,sizeof(Matrix_4x4));
    m[0][0] = (float)1;
    m[1][1] = (float)1;
    m[2][2] = (float)1;
    m[3][3] = (float)1;
}

//--- Matrix-Vector multiply ------------------------------------------------

void Mat_Mul_1x4(Vector_3D v1, Vector_3D v2, Matrix_4x4 m)
{
    v1.x = v2.x * m[0][0] + v2.y * m[1][0] + v2.z * m[2][0] + m[3][0];
    v1.y = v2.x * m[0][1] + v2.y * m[1][1] + v2.z * m[2][1] + m[3][1];
    v1.z = v2.x * m[0][2] + v2.y * m[1][2] + v2.z * m[2][2] + m[3][2];
}

//--- Matrix-Matrix multiply ------------------------------------------------

void Mat_Mul_4x4(Matrix_4x4 m1, Matrix_4x4 m2, Matrix_4x4 r)
{
    for (int x = 0; x < 4; x++) {
       for (int y = 0; y < 4; y++) {
          r[x][y] = 0;
          for (int k = 0; k < 4; k++) r[x][y] += (m1[k][y] * m2[x][k]);
       }
    }
}
 
//--- Matrix-Vector Concatenation -------------------------------------------

void Mat_Concatenate_1x4(Vector_3D v, Matrix_4x4 m)
{
    float x0 = v.x;
    float y0 = v.y;
    float z0 = v.z;

    v.x = x0 * m[0][0] + y0 * m[1][0] + z0 * m[2][0] + m[3][0];
    v.y = x0 * m[0][1] + y0 * m[1][1] + z0 * m[2][1] + m[3][1];
    v.z = x0 * m[0][2] + y0 * m[1][2] + z0 * m[2][2] + m[3][2];
}

//--- Matrix-Matrix Concatenation -------------------------------------------

void Mat_Concatenate_4x4(Matrix_4x4 md, Matrix_4x4 ms)
{
    Matrix_4x4 tmp;

    Mat_Mul_4x4(md, ms, tmp);
    memcpy(md, tmp, sizeof(Matrix_4x4));
}

//--- Print 4x4 Matrix ------------------------------------------------------

void Mat_Print_4x4(Matrix_4x4 m)
{
    for (int i = 0; i < 4; i++)
        printf("%8.2f, %8.2f, %8.2f, %8.2f\n", m[0][i], m[1][i], m[2][i], m[3][i]);
}


