//****************************************************************************
//
// File         : memory.cpp
//
// Description  : Simple memory management functions.
//                Written by Lars Andersson (lars_and@algonet.se), 1998
//
//****************************************************************************

#include <malloc.h>
#include <string.h>

#include "memory.h"

inline int  Findex(void *address);

int         nr_ptrs = 0;
void        *memory_ptrs[256];


void *AllocMem(unsigned long size)
{
    if (NULL == (memory_ptrs[nr_ptrs] = malloc(size))) return(NULL);
    memset(memory_ptrs[nr_ptrs], 0, size);
    return(memory_ptrs[nr_ptrs++]);
}


void *ReAllocMem(void *address, unsigned long size)
{
    int     index;
    void    *newptr;

    if (address == NULL) return(AllocMem(size));
    if (size == 0) {
        FreeMem(address);
        return(NULL);
    }

    index = Findex(address);
    if (index == -1) return(NULL);

    if (NULL == (newptr = realloc(memory_ptrs[index], size))) return(NULL);

    memory_ptrs[index] = newptr;
    return(newptr);
}

                           
void FreeMem(void *address)
{
    int i, index;

    index = Findex(address);
    if (index == -1) return;

    free(memory_ptrs[index]);
    nr_ptrs--;

    for (i = index; i < nr_ptrs; i++) {
        memory_ptrs[i] = memory_ptrs[i + 1];
    }
}


void FreeAll(void)
{
    int i;

    for (i = 0; i < nr_ptrs; i++) {
        free(memory_ptrs[i]);
    }
}


inline int Findex(void *address)
{
    int i;
    for (i = 0; i < nr_ptrs; i++) {
        if (address == memory_ptrs[i]) return(i);
    }
    return(-1);
}


