//****************************************************************************
//
// File         : pwd.h
//
// Description  : PWD (Portal Word Data) file format structures
//                Written by Lars Andersson (lars_and@algonet.se), 1998
//
//****************************************************************************

#ifndef PWD_H
#define PWD_H

//--- Flag bits -------------------------------------------------------------

#define PWDHEAD_PALETTE 1
#define PWDHEAD_EXTTEXTURES 2

#define PWDPOLY_PORTAL 1
#define PWDPOLY_CROSSABLE 2
#define PWDPOLY_TEXTURED 4
#define PWDPOLY_GOURAUD 8

//--- Structures ------------------------------------------------------------

// Turn on structure packing

#ifdef COMP_WCC
    #pragma pack (push);
    #pragma pack (1);         // Watcom C++ : No structure member alignement
#endif

#if defined(COMP_DJGPP) || defined(COMP_GCC)
    #pragma pack(1)           // DJGPP / GCC : No structure member alignement
#endif

typedef struct {
    char            worldname[32];
    int             texture_offset;
    short           nr_objects, nr_textures;
    short           flags;
} PWD_Header;

typedef struct {
    short           id, nr_sectors, nr_vertices, nr_polygons;
    short           flags;
} PWD_ObjectHeader;

typedef struct {
    short           id, nr_vertices, nr_polygons, nr_portals;
    short           flags;
} PWD_SectorHeader;

typedef struct {
    short           id, xsize, ysize, bpp;
    short           flags;
} PWD_TextureHeader;

typedef struct {
    int             attribute;
    short           flags;
    short           points;
} PWD_PolyHeader;

typedef struct {
    float           x,y,z;
} PWD_Vertex;

// Turn of structure packing

#ifdef COMP_WCC
    #pragma pack (pop);
#endif

#if defined(COMP_DJGPP) || defined(COMP_GCC)
    #pragma pack()
#endif

//--- Function declarations -------------------------------------------------

int     Load_PWDWorld(char *filename);
void    Free_World(void);

int     Read_Object(Object *object_p);
int     Read_Sector(Sector *sector_p);

#endif
