//****************************************************************************
//
// File         : screen.h
//
// Description  : Screen mode structures and function defs.
//                Written by Lars Andersson (lars_and@algonet.se), 1998
//
//****************************************************************************

#ifndef SCREEN_H
#define SCREEN_H

#ifndef VIDEO_H
#include "video.h"
#endif

//--- Misc definitions ------------------------------------------------------

#define MODEINFO_VGA 1
#define MODEINFO_XVGA (1<<2)
#define MODEINFO_SVGA (1<<3)
#define MODEINFO_X11 (1<<4)
#define MODEINFO_FLIPABLE (1<<5)
#define MODEINFO_PAGEFLIP (1<<6)
#define MODEINFO_CLEAR (1<<7)
#define MODEINFO_WVBL (1<<8)

#define SCREENMODE_SET (1<<0)
#define SCREENMODE_BUFFER (1<<5)

//--- Structure definitions -------------------------------------------------

typedef struct {
    short       xres, yres, size, code, flags;
} ModeInfo;

typedef struct {
    int         flags, size, page;
    int         xres, yres;
    int         width, height;
    float       top_clip, bottom_clip, right_clip, left_clip;
    int         bytes_per_row, bytes_per_frame;
    float       xcenter, ycenter;
    float       pmulx, pmuly, pmulx_zclip, pmuly_zclip, pmul_ratio;
    int         *ymultab;
    Clip        *cclip;
    char        *memory, *buffer, *videomem, *lfb;
    void        (*Show)(void);
    void        (*BClear)(void);
    void        (*VClear)(void);
    int         videomemory;
    int         nr_modes;
    ModeInfo    *modelist, *currentmode;
    char        *vendorname;
    char        *cardname;
} ScrMode;

#ifdef COMP_WCC
    #pragma pack (push);
    #pragma pack (1);         // Watcom C++ : No structure member alignement
#endif

#if defined(COMP_GCC) || defined(COMP_DJGPP)
    #pragma pack(1)
#endif

typedef struct {
   char         id_length;
   char         color_map_type;
   char         image_type;
   short        first_color_map_entry;
   short        color_map_length;
   char         color_map_entry_size;
   short        image_x_origin;
   short        image_y_origin;
   short        image_width;
   short        image_height;
   char         bits_per_pixel;
   char         image_descriptor_bits;
} TGA_Header;

#ifdef COMP_WCC
    #pragma pack (pop);
#endif

#if defined(COMP_GCC) || defined(COMP_DJGPP)
    #pragma pack()
#endif

//--- Function declarations -------------------------------------------------

int     Init_Screen();
int     Set_ScreenMode(int xres, int yres);
int     Set_PageFlip(int toggle);
void    Flip_Buffer(void);
int     Save_TGA(void);
void    Save_ScreenState(void);
void    Restore_ScreenState(void);
void    Init_ModeConstants(int mode);
void    Close_ScreenMode(void);
void    Close_Screen();

void    Shrink_Screen(void);
void    Enlarge_Screen(void);


#endif
