//****************************************************************************
//
// File         : input.cpp
//
// Description  : Linux-SVGALib/GCC keyboard/mouse routines...
//                Written by Lars Andersson (lars_and@algonet.se), 1998
//
//****************************************************************************

#define INPUT

#include <stdio.h>
#include <vga.h>
#include <vgakeyboard.h>
#include <vgamouse.h>

#include "input.h"


char transtab[128] = {0, 0, 49, 50, 51, 52, 53, 54, 55, 56, 57, 48, 43, 0, 8,
                0, 81, 87, 69, 82, 84, 89, 85, 73, 79, 80, 0, 0, 13, 0,
                65, 83, 68, 70, 71, 72, 74, 75, 76, 58, 39, 0, 0, 0,
                90, 88, 67, 86, 66, 78, 77, 44, 46, 45, 0, 0, 0, 32,
                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 55, 56, 57, 45,
                52, 53, 54, 43, 49, 50, 51, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

int             mousex, mousey, mousedx, mousedy, mousebuttons;
volatile char 	keys[256];
volatile    	char hit, lastcode, special;

void Key_Handler(int scancode, int pressed)
{
    if (pressed) {
		keys[scancode] = 1;
		hit = 1;
        lastcode = scancode; 
    }
    else {
    	keys[scancode] = 0;
    }
}
 
//--- Init keyboard stuff ----------------------------------------------------

void Init_Keyboard(void)
{
    keyboard_init();
    keyboard_clearstate();
    keyboard_seteventhandler(Key_Handler);
}

//--- Close down keyboard stuff ----------------------------------------------

void Close_Keyboard(void)
{
    keyboard_close();
}

//--- Initialize mouse ------------------------------------------------------

int Init_Mouse(void)
{
    int mousetype, status;

    mousetype = vga_getmousetype();
    status = mouse_init("/dev/mouse", mousetype, MOUSE_DEFAULTSAMPLERATE);
    mouse_setxrange(-100000, 100000);
    mouse_setyrange(-100000, 100000);
    return(!status);
}

void Close_Mouse(void)
{
    mouse_close();
}


//--- Update Input ------------------------------------------------------------

void Update_Input(void)
{
    int x, y, b;    
	
	keyboard_update();
	mouse_update();
    x = mouse_getx();
    y = mouse_gety();

    mousedx = x - mousex;
    mousedy = y - mousey;
    mousex = x;
    mousey = y;

    b = mouse_getbutton();
    mousebuttons = 0;
    if (b & MOUSE_LEFTBUTTON) mousebuttons |= LEFT_MBUTTON;
    if (b & MOUSE_RIGHTBUTTON) mousebuttons |= RIGHT_MBUTTON; 
}


//--- Returns TRUE if a key has been hit since last call --------------------

char KeyHit(void)
{
    if (hit) {
        hit=FALSE;
        return TRUE;
    } else return FALSE;
}

//--- Returns TRUE if key has been hit and waits until it's released --------

char TestKey(int key)
{
    if (keys[key]) {
        while(keys[key]) keyboard_update();
        return TRUE;
    } else return FALSE;
}

//--- Wait for key and return ascii code ------------------------------------

char GetChar(void) {

    char    ch;

    while (!hit) keyboard_update();
    hit = 0;
    ch = transtab[lastcode];
    if (ch != 0) keys[lastcode] = 0;
    return(ch);
}


//--- Set mouse sensitivity -------------------------------------------------

void Set_MouseSensitivity(int s, int d)
{
}
