//****************************************************************************
//
// File         : video.cpp
//
// Description  : Linux-SVGALib/GCC low level videomode functions.
//                Written by Lars Andersson (lars_and@algonet.se), 1998
//
//****************************************************************************

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <vga.h>

#include "tweaked.h"
#include "main.h"
#include "video.h"

short           oldmode = 0;

int             tweaked_rowlen, tweaked_ysize;

ModeInfo        standard_svgalibmodes[3] = {{320, 200, 0, 0, MODEINFO_VGA},
                    {320, 240, 0, 1, MODEINFO_XVGA},
                    {320, 400, 0, 2, MODEINFO_XVGA}};

//--- Init video system ------------------------------------------------------

int Init_Video(void)
{
    int         svga_modes = 0;

	vga_init();

    engine.screen.vendorname = "UNKNOWN";    // Assume standard 128K VGA card
    engine.screen.cardname = "STANDARD VGA";
    engine.screen.videomemory = 131072;       

    engine.screen.modelist = (ModeInfo *)(malloc((3) * sizeof(ModeInfo)));
    if (!engine.screen.modelist) return(FALSE);

    // Copy Standard VGA modes...

    memcpy(engine.screen.modelist, standard_svgalibmodes, 3 * sizeof(ModeInfo));

    engine.screen.nr_modes = 3;
//    realloc(video.modelist, 3 * sizeof(ModeInfo));
    oldmode = vga_getcurrentmode();    
   
    return(TRUE);
}

//--- Close down video system -----------------------------------------------

void Close_Video()
{
	if (engine.screen.modelist) free(engine.screen.modelist);
    vga_setmode(oldmode);
}

//--- Set SVGALib video mode -------------------------------------------------

int Set_VideoMode(int mode)
{
    if (mode == 0) {
    	if (-1 == vga_setmode(G320x200x256)) return(FALSE);
        engine.screen.Show = Copy_Buffer;
        engine.screen.BClear = Clear_Buffer;
        engine.screen.VClear = Clear_VGA;
    } else {
    	tweaked_rowlen = 80;
    	if (mode == 1) { 
    		if (-1 == vga_setmode(G320x240x256)) return(FALSE);
  			tweaked_ysize = 240;
  		}
  		if (mode == 2) { 
			if (-1 == vga_setmode(G320x400x256)) return(FALSE);
        	tweaked_ysize = 400;
        }
        engine.screen.Show = Copy_XVGA;
        engine.screen.BClear = Clear_Buffer;
        engine.screen.VClear = Clear_XVGA;
   	}

   	engine.screen.lfb = (char *)(vga_getgraphmem());
   	return(TRUE);
}


//--- Close down current video mode -----------------------------------------

void Close_VideoMode()
{
}


//--- Copy linear frame buffer to video memory ------------------------------

void Copy_Buffer(void)
{
    char        *source, *dest;

    source = engine.screen.buffer;
    dest = engine.screen.videomem +
        ((engine.screen.yres - engine.screen.height) >> 1) * engine.screen.xres +
        ((engine.screen.xres - engine.screen.width) >> 1);

    int dwords = engine.screen.width >> 2;

    if (engine.screen.currentmode->flags & MODEINFO_WVBL) Wait_Retrace();

    if (engine.screen.currentmode->flags & MODEINFO_CLEAR) {
        for (int i = 0; i < engine.screen.height; i++) {
            Dword_CopyClear(dest, source, 0, dwords);
            source += engine.screen.width;
            dest += engine.screen.xres;
        }
    } else {
        for (int i = 0; i < engine.screen.height; i++) {
            Dword_Copy(dest, source, dwords);
            source += engine.screen.width;
            dest += engine.screen.xres;
        }
    }
}
/*
//--- Flip linear framebuffer pages -----------------------------------------

void Flip_Buffer(void)
{
    if (engine.screen.currentmode->flags & MODEINFO_WVBL) Wait_Retrace();
    if(engine.screen.page == 0) {
        engine.screen.page = 1;
        Set_DisplayStart(0, engine.screen.yres);
    } else {
        engine.screen.page = 0;
        Set_DisplayStart(0, 0);
    }
    char *tmp = engine.screen.buffer;
    engine.screen.buffer = engine.screen.videomem;
    engine.screen.videomem = tmp;

    if (engine.screen.currentmode->flags & MODEINFO_CLEAR) {
        char *dest = engine.screen.buffer +
            (int)(engine.screen.top_clip) * engine.screen.xres +
            (int)(engine.screen.left_clip);

        for (int i = 0; i < engine.screen.height; i++) {
            Dword_Fill(dest, 0, engine.screen.width >> 2);
            dest += engine.screen.xres;
        }
    }
}
*/
//--- Clear linear frame buffer ---------------------------------------------

void Clear_Buffer(void)
{
    char *dest = engine.screen.buffer +
        (int)(engine.screen.top_clip) * engine.screen.xres +
        (int)(engine.screen.left_clip);

    for (int i = 0; i < engine.screen.height; i++) {
        Dword_Fill(dest, 0, engine.screen.width >> 2);
        dest += engine.screen.xres;
    }

}

void Clear_VGA(void)
{
    Dword_Fill(engine.screen.videomem, 0, (engine.screen.xres * engine.screen.yres) >> 2);
    if (engine.screen.currentmode->flags & MODEINFO_PAGEFLIP)
        Dword_Fill(engine.screen.buffer, 0, (engine.screen.xres * engine.screen.yres) >> 2);
}


//--- Copy buffer to chained video memory -----------------------------------

void Copy_XVGA(void)
{
    int x = (engine.screen.xres - engine.screen.width) >> 1;
    int y = (engine.screen.yres - engine.screen.height) >> 1;

    Tweaked_CopyBlock(x, y, engine.screen.width,
        engine.screen.height, engine.screen.buffer);

    if (engine.screen.currentmode->flags & MODEINFO_CLEAR)
        Dword_Fill(engine.screen.buffer, 0, (engine.screen.width * engine.screen.height) >> 2);
}

void Clear_XVGA(void)
{
    Tweaked_ClearBlock(0, 0, engine.screen.xres, engine.screen.yres, 0);
}


//--- Wait for vertical retrace ---------------------------------------------

void Wait_Retrace(void)
{
	vga_waitretrace();	
}

//--- Set VGA palette -------------------------------------------------------

void Set_Palette(char *palette)
{
    asm(" cld                   \n\t"
        " movl  $0x3c8, %%edx   \n\t"
        " xorl  %%eax, %%eax    \n\t"
        " outb  %%al, %%dx      \n\t"
        " incl  %%edx           \n\t"
        " movl  $768, %%ecx     \n\t"
        " rep                   \n\t"
        " outsb                 \n\t"
        :
        : "S" (palette)
        : "%eax", "%ecx", "%edx", "%esi");
}

//--- Sets scanline width ----------------------------------------------------

int Set_ScanWidth(long width)
{
    return(TRUE);
}

//--- Sets display start location --------------------------------------------

int Set_DisplayStart(long x, long y)
{
    return(TRUE);
}
/*
//--- If possible, toggle pageflipping on/off --------------------------------

int Set_PageFlip(int toggle)
{

    if (toggle) {
        if ((video.memory < (engine.screen.bytes_per_frame * 2)) ||
            (!(video.flags & VIDEO_FLIPABLE))) return(FALSE);

        engine.screen.videomem = video.videomem;
        engine.screen.buffer = video.videomem + engine.screen.xres * engine.screen.yres;
        engine.screen.page = 0;
        video.Show = Flip_Buffer;
        engine.screen.flags |= SCREENMODE_PAGEFLIP;
    }
    else {
        if (!(engine.screen.flags & SCREENMODE_PAGEFLIP)) return(FALSE);
        engine.screen.videomem = video.videomem;
        Dword_Copy(video.videomem, engine.screen.videomem, engine.screen.bytes_per_frame >> 2);
        engine.screen.buffer = engine.screen.memory;
        video.Show = Copy_Buffer;
        engine.screen.flags &= ~SCREENMODE_PAGEFLIP;
        Set_DisplayStart(0, 0);
    }

    return(TRUE);
}
*/
//--- Clear block on chained video memory ------------------------------------

void ClearBlockAsm(int xsize, int ysize, int color, char *dest);

void ClearBlockAsm(int xsize, int ysize, int color, char *dest)
{
    asm(" pushl %%edx           \n\t"
        " movw  $0x3c4, %%dx    \n\t"
        " movl  %%eax, %%ecx    \n\t"
        " movl  $0xf02, %%eax   \n\t"
        " outw  %%ax, %%dx      \n\t"
        " movb  %%bl, %%bh      \n\t"
        " movl  %%ebx, %%eax    \n\t"
        " shll  $16, %%ebx      \n\t"
        " popl   %%edx          \n\t"
        " movw  %%ax, %%bx      \n\t"
        " movl  %%ecx, %%eax    \n\t"
        " movl  tweaked_rowlen, %%esi \n\t"
        " addl  %%eax, %%eax    \n\t"
        " subl  %%eax, %%esi    \n\t"
        " xorb  %%ah, %%ah      \n\t"
        " shrl  $1, %%ecx       \n\t"
        " jnc   1f              \n\t"
        " movb  $1, %%ah        \n\t"
    "1:                         \n\t"       // YLoop
        " movb  %%cl, %%al      \n\t"
    "2:                         \n\t"       // XLoop
        " movl  %%ebx, (%%edi)  \n\t"
        " addl  $4, %%edi       \n\t"
        " decb  %%al            \n\t"
        " jnz   2b              \n\t"
        " orb   %%ah, %%ah      \n\t"
        " jz    3f              \n\t"
        " movw  %%bx, (%%edi)   \n\t"
        " addl  $2, %%edi       \n\t"
    "3:                         \n\t"       // Even
        " addl  %%esi, %%edi    \n\t"
        " dec   %%edx           \n\t"
        " jnz   1b               "
        :
        : "a" (xsize), "d" (ysize), "b" (color), "D" (dest)
        : "%eax", "%ebx", "%ecx", "%edx", "%edi", "%esi");
}

void Tweaked_ClearBlock(int x, int y, int xsize, int ysize, int color)
{
    ClearBlockAsm(xsize >> 3, ysize, color, engine.screen.videomem +
        y * tweaked_rowlen + (x >> 2));
}

//--- Copy block to chained video memory ------------------------------------

void CopyBlockAsm(int xsize, int ysize, char *dest, char *source);

void CopyBlockAsm(int xsize, int ysize, char *dest, char *source)
{
    asm(" movl  %%eax, %%ebp    \n\t"
        " movl  %%eax, %%ecx    \n\t"
        " shrl  $1, %%eax       \n\t"
        " shll  $3, %%ebp       \n\t"
        " movb  %%al, %%ch      \n\t"
        " subl  $4, %%ebp       \n\t"
    "1:                         \n\t"       // YLoop
        " xorb  %%cl, %%cl      \n\t"
    "2:                         \n\t"       // PLoop
        " movl  $0x3c4, %%edx   \n\t"
        " movl  $0x102, %%eax   \n\t"
        " shlb  %%cl, %%ah      \n\t"
        " pushl %%edi           \n\t"
        " outw  %%ax, %%dx      \n\t"
        " pushl %%esi           \n\t"
        " movb  %%ch, %%dl      \n\t"
     "3:                        \n\t"       // XLoop
        " movb  12(%%esi), %%ah \n\t"
        " movb  8(%%esi), %%al  \n\t"
        " shll  $16, %%eax      \n\t"
        " movb  4(%%esi), %%ah  \n\t"
        " movb  (%%esi), %%al   \n\t"
        "                       \n\t"
        " movl  %%eax, (%%edi)  \n\t"
        " addl  $16, %%esi      \n\t"
        " addl  $4, %%edi       \n\t"
        " decb  %%dl            \n\t"
        " jnz   3b              \n\t"
        "                       \n\t"
        " popl  %%esi           \n\t"
        " incb  %%cl            \n\t"
        " incl  %%esi           \n\t"
        " popl  %%edi           \n\t"
        " cmpb  $4, %%cl        \n\t"
        " jne   2b              \n\t"
        "                       \n\t"
        " addl  tweaked_rowlen, %%edi \n\t"
        " addl  %%ebp, %%esi    \n\t"
        "                       \n\t"
        " decl  %%ebx           \n\t"
        " jnz   1b                  "
        :
        : "a" (xsize), "b" (ysize), "D" (dest), "S" (source)
        : "%eax", "%ebx", "%ecx", "%edx", "%edi", "%esi", "%ebp");
}

void Tweaked_CopyBlock(int x, int y, int xsize, int ysize, char *source)
{
    CopyBlockAsm(xsize >> 3, ysize, engine.screen.videomem +
        y * tweaked_rowlen + (x >> 2), source);
}


