//****************************************************************************
//
// File         : text.cpp
//
// Description  : Linear framebuffer text routines.
//                Written by Lars Andersson (lars_and@algonet.se), 1997-1998
//
//****************************************************************************

#include <stdio.h>
#include <stdlib.h>

#include "main.h"
#include "text.h"
#include "sfont.h"

long                scrwidth, textx, texty, textpos, txtcolor;
char                tcolor, bcolor;

//--- Offsets into font data ------------------------------------------------

//            !    "    # $    %    & '    (    )    *    +    ,    -
int cvttab[]={2296,2856,0,2968,2912,0,2688,2744,2800,2520,2408,2240,2464,
//              .    /    0    1    2    3    4    5    6    7    8    9
                2184,2576,1624,1680,1736,1792,1848,1904,1960,2016,2072,2128,
//              :    ; < =    > ?    @    A B  C   D   E   F   G   H   I
                2632,0,0,3080,0,2352,3136,0,56,112,168,224,280,336,392,448,
//              J   K   L   M   N   O   P   Q   R   S    T    U    V    W
                504,560,616,672,728,784,840,896,952,1008,1064,1120,1176,1232,
//              X    Y    Z
                1288,1344,1400};


#ifdef COMP_WCC

//--- Watcom pragma ---------------------------------------------------------

void PutChar(int ch);

#pragma aux PutChar=" cmp   al, 32               ",\
                    " jb    NoChar               ",\
                    " je    Space                ",\
                    " mov   esi, offset sfont    ",\
                    " sub   eax, 33              ",\
                    " mov   edi, [textpos]       ",\
                    " mov   cl, 8                ",\
                    " add   esi, [eax*4+cvttab]  ",\
                    " mov   ebx, txtcolor        ",\
                "YLoop:                          ",\
                    " mov   ch, 7                ",\
                "XLoop:                          ",\
                    " mov   al, [esi]            ",\
                    " inc   esi                  ",\
                    " cmp   al, 1                ",\
                    " ja    Color                ",\
                    " jb    NoPix                ",\
                    " mov   [edi], bl            ",\
                    " inc   edi                  ",\
                    " dec   ch                   ",\
                    " jnz   XLoop                ",\
                    " jmp   XDone                ",\
                "Color:                          ",\
                    " mov   [edi], bh            ",\
                "NoPix:                          ",\
                    " inc   edi                  ",\
                    " dec   ch                   ",\
                    " jnz   XLoop                ",\
                "XDone:                          ",\
                    " add   edi, scrwidth        ",\
                    " sub   edi, 7               ",\
                    " dec   cl                   ",\
                    " jnz   YLoop                ",\
                "Space:                          ",\
                    " add   textpos, 7           ",\
                "NoChar:                         ",\
                    parm[eax] modify[eax ebx ecx esi edi];

#endif


#ifdef COMP_DJGPP

//--- DJGPP inline asm ------------------------------------------------------

void PutChar(int ch);

void PutChar(int ch)
{
  asm (" cmpb    $32, %%al              \n\t"
       " jb      7f                     \n\t"
       " je      6f                     \n\t"
       " movl    $_sfont, %%esi         \n\t"
       " subl    $33, %%eax             \n\t"
       " movl    _textpos, %%edi        \n\t"
       " movb    $8, %%cl               \n\t"
       " addl    _cvttab(,%%eax,4), %%esi \n\t"
       " movl    _txtcolor, %%ebx       \n\t"
   "1:                                  \n\t"       // YLoop
       " movb    $7, %%ch               \n\t"
   "2:                                  \n\t"       // XLoop
       " movb    (%%esi), %%al          \n\t"
       " incl    %%esi                  \n\t"
       " cmpb    $1, %%al               \n\t"
       " ja      3f                     \n\t"
       " jb      4f                     \n\t"
       " movb    %%bl, (%%edi)          \n\t"
       " incl    %%edi                  \n\t"
       " decb    %%ch                   \n\t"
       " jnz     2b                     \n\t"
       " jmp     5f                     \n\t"
   "3:                                  \n\t"       // Color
       " movb    %%bh, (%%edi)          \n\t"
   "4:                                  \n\t"       // NoPix
       " incl    %%edi                  \n\t"
       " decb    %%ch                   \n\t"
       " jnz     2b                     \n\t"
   "5:                                  \n\t"       // XDone
       " addl    _scrwidth, %%edi       \n\t"
       " subl    $7, %%edi              \n\t"
       " decb    %%cl                   \n\t"
       " jnz     1b                     \n\t"
   "6:                                  \n\t"       // Space
       " addl    $7, _textpos           \n\t"
   "7:                                  "           // Nochar
       :
       : "a" (ch)
       : "%ebx", "%ecx", "%edx", "%esi", "%edi");
}

#endif


#ifdef COMP_GCC

//--- GCC inline asm --------------------------------------------------------

void PutChar(int ch);

void PutChar(int ch)
{
  asm (" cmpb    $32, %%al              \n\t"
       " jb      7f                     \n\t"
       " je      6f                     \n\t"
       " movl    $sfont, %%esi         \n\t"
       " subl    $33, %%eax             \n\t"
       " movl    textpos, %%edi        \n\t"
       " movb    $8, %%cl               \n\t"
       " addl    cvttab(,%%eax,4), %%esi \n\t"
       " movl    txtcolor, %%ebx       \n\t"
   "1:                                  \n\t"       // YLoop
       " movb    $7, %%ch               \n\t"
   "2:                                  \n\t"       // XLoop
       " movb    (%%esi), %%al          \n\t"
       " incl    %%esi                  \n\t"
       " cmpb    $1, %%al               \n\t"
       " ja      3f                     \n\t"
       " jb      4f                     \n\t"
       " movb    %%bl, (%%edi)          \n\t"
       " incl    %%edi                  \n\t"
       " decb    %%ch                   \n\t"
       " jnz     2b                     \n\t"
       " jmp     5f                     \n\t"
   "3:                                  \n\t"       // Color
       " movb    %%bh, (%%edi)          \n\t"
   "4:                                  \n\t"       // NoPix
       " incl    %%edi                  \n\t"
       " decb    %%ch                   \n\t"
       " jnz     2b                     \n\t"
   "5:                                  \n\t"       // XDone
       " addl    scrwidth, %%edi       \n\t"
       " subl    $7, %%edi              \n\t"
       " decb    %%cl                   \n\t"
       " jnz     1b                     \n\t"
   "6:                                  \n\t"       // Space
       " addl    $7, textpos           \n\t"
   "7:                                  "           // Nochar
       :
       : "a" (ch)
       : "%ebx", "%ecx", "%edx", "%esi", "%edi");
}

#endif



void Set_TextColor(char f, char b)
{
    txtcolor = (f << 8) + b;
}

void PutCharXY(int x, int y, char c)
{
    scrwidth = engine.screen.bytes_per_row;
    textpos = (long)(engine.screen.buffer + y * engine.screen.bytes_per_row + x);
    PutChar(c);
}

void PutStringXY(int x, int y, char *str)
{
    scrwidth = engine.screen.bytes_per_row;
    textpos = (long)(engine.screen.buffer + y * engine.screen.bytes_per_row + x);
    while(*str != (char)0) PutChar(*str++);
}

void PutIntXY(int x, int y, int value)
{
    char str[16];
    sprintf(str, "%d", value);
    PutStringXY(x,y, str);
}
