//****************************************************************************
//
// File         : vector.cpp
//
// Description  : Misc 2D/3D vector functions.
//                Written by Lars Andersson (lars_and@algonet.se), 1998
//
//****************************************************************************

#include <math.h>

#include "main.h"

//--- Make vector from two vertices -----------------------------------------

void Make_Vector_3D(Vertice_3D *v1, Vertice_3D *v2, Vector_3D *u)
{
    u->x = v1->x - v2->x;
    u->y = v1->y - v2->y;
    u->z = v1->z - v2->z;
}

//--- Calculate lenght of a vector ------------------------------------------

float Vector_Magnitude(Vector_3D *u)
{
    return(sqrt(u->x * u->x + u->y * u->y + u->z * u->z));
}

//--- Calculate distance between two 3D points ------------------------------

float Vertice_Distance(Vertice_3D *u, Vertice_3D *v)
{
    double tmp = (u->x-v->x)*(u->x-v->x) + (u->y-v->y)*(u->y-v->y) +
        (u->z-v->z)*(u->z-v->z);
    return((float)sqrt(tmp));
}

//--- Normalize 3d vector ---------------------------------------------------

void Normalize_Vector(Vector_3D *u)
{

    double length = Vector_Magnitude(u);
    if (length != 0) {
        u->x /= length;
        u->y /= length;
        u->z /= length;
    }
}

//--- Calculate cross-product -----------------------------------------------

void Cross_Product(Vector_3D *u, Vector_3D *v, Vector_3D *cp)
{
    cp->x = (u->y * v->z) - (u->z * v->y);
    cp->y = (u->z * v->x) - (u->x * v->z);
    cp->z = (u->x * v->y) - (u->y * v->x);
}

//--- Calculate dot-product -------------------------------------------------

float Dot_Product(Vector_3D *u, Vector_3D *v)
{
    return((u->x * v->x) + (u->y * v->y) + (u->z * v->z));
}

//--- Determine at which side of a line the point x,y lies ------------------

int Which_Side_2D(Line_2D *v, float x, float y)
{
    float s = (v->y1 - y)*(v->x2 - v->x1) - (v->x1 - x)*(v->y2 - v->y1);
    if (s < 0) return -1;
    else if (s > 0) return 1;
    else return 0;
}

//--- Calculate line intersection -------------------------------------------

int Seg_Line_Intersect(Line_2D *s, Line_2D *l, Vertice_2DD *isect)
{
    float   denom;
    float   r;

    denom = (s->x2 - s->x1) * (l->y2 - l->y1) -
        (s->y2 - s->y1) * (l->x2 - l->x1);
    if (fabs(denom) < VERY_SMALL) return(FALSE);

    r = ((s->y1 - l->y1) * (l->x2 - l->x1) -
        (s->x1 - l->x1) * (l->y2 - l->y1)) / denom;

    if ((r < -VERY_SMALL) || (r > 1+VERY_SMALL)) return(FALSE);

    isect->x = s->x1 + r * (s->x2 - s->x1);
    isect->y = s->y1 + r * (s->y2 - s->y1);

    return(TRUE);
}
