//****************************************************************************
//
// File         : vector.h
//
// Description  : Misc 2D/3D vector structures.
//                Written by Lars Andersson (lars_and@algonet.se), 1998
//
//****************************************************************************

#ifndef VECTOR_H
#define VECTOR_H

typedef struct {
    int         x, y;
} Vertice_2DI;

typedef struct {
    float       x, y;
} Vertice_2DF, Vector_2D;

typedef struct {
    double      x, y;
} Vertice_2DD;

typedef struct {
    float       x, y, z;
} Vertice_3D, Vector_3D;

typedef struct {
    float       x1,x2,y1,y2;
} Line_2D;

inline void Vector_Add(Vector_3D *d, Vector_3D *s);
inline void Vector_Add(Vector_3D *d, Vector_3D *s)
{
    d->x += s->x;
    d->y += s->y;
    d->z += s->z;
}

inline void Vector_Sub(Vector_3D *d, Vector_3D *s);
inline void Vector_Sub(Vector_3D *d, Vector_3D *s)
{
    d->x -= s->x;
    d->y -= s->y;
    d->z -= s->z;
}

void    Make_Vector_3D(Vertice_3D *v1, Vertice_3D *v2, Vector_3D *u);
float   Vector_Magnitude(Vector_3D *u);
float   Vertice_Distance(Vertice_3D *u, Vertice_3D *v);
void    Normalize_Vector(Vector_3D *u);
void    Cross_Product(Vector_3D *u, Vector_3D *v, Vector_3D *normal);
float   Dot_Product(Vector_3D *u, Vector_3D *v);
int     Which_Side_2D(Line_2D *v, float x, float y);
int     Seg_Line_Intersect(Line_2D *s, Line_2D *l, Vertice_2DD *isect);

#endif

