//***************************************************************************
//
// File         : input.cpp
//
// Description  : DOS/Watcom Protected mode keyboard/mouse routines...
//                Written by Lars Andersson (lars_and@algonet.se), 1997
//
//***************************************************************************

#define INPUT

#include <stdio.h>
#include <conio.h>
#include <dos.h>

#include "input.h"

#define KEY_INT 0x9
#define MOUSE_INT 0x33

void (__interrupt __far *oldint_09)();

char transtab[128] = {0, 0, 49, 50, 51, 52, 53, 54, 55, 56, 57, 48, 43, 0, 8,
                0, 81, 87, 69, 82, 84, 89, 85, 73, 79, 80, 0, 0, 13, 0,
                65, 83, 68, 70, 71, 72, 74, 75, 76, 58, 39, 0, 0, 0,
                90, 88, 67, 86, 66, 78, 77, 44, 46, 45, 0, 0, 0, 32,
                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 55, 56, 57, 45,
                52, 53, 54, 43, 49, 50, 51, 0, 0};

int         mousex, mousey, mousedx, mousedy, mousebuttons;
volatile    char keys[128];
volatile    char hit, lastcode, special;
char        keyboard_installed = FALSE;

//--- The keyboard interrupt handler ----------------------------------------

void __far __interrupt KeyHandler()
{
    char code = (char)inp(0x60);

       if (code < 128) {
            keys[code] = 1;
            hit = 1;
            if (special) {
                special = FALSE;
                lastcode = 0;
            } else {
                lastcode = code;
            }
        }
        else {
            if (special) special = FALSE;
            if (code == 224) special = TRUE;
            else keys[code-128] = 0;
        }

    code = (char)inp(0x61);
    outp(0x61, code | 0x80);
    outp(0x20, 0x20);
}

//--- Install our new protected mode Keyboard ISR ---------------------------

void Init_Keyboard(void)
{
    if (!keyboard_installed) {
        for (int i = 0; i < 128; i++) keys[i] = FALSE;
        oldint_09 = _dos_getvect(KEY_INT);
        _dos_setvect(KEY_INT, KeyHandler);
        keyboard_installed = TRUE; hit = special = FALSE;
    }
}

//--- Remove Keyboard ISR ---------------------------------------------------

void Close_Keyboard(void)
{
    if (keyboard_installed) {
        _dos_setvect(KEY_INT, oldint_09);
        keyboard_installed = FALSE;
    }
}

//--- Initialize mouse ------------------------------------------------------

int Init_Mouse(void)
{
    union REGS regs;

    regs.x.eax = 0;
    int386(MOUSE_INT, &regs, &regs);
    return(regs.x.eax);
}

//--- Close Mouse -----------------------------------------------------------

void Close_Mouse(void)
{
}

//--- Update Input ----------------------------------------------------------

void Update_Input(void)
{
    union REGS regs;

    regs.x.eax = 3;
    int386(MOUSE_INT, &regs, &regs);
    mousebuttons = regs.w.bx;

    mousex = (int)regs.w.cx;
    mousey = (int)regs.w.dx;

    regs.x.eax = 0xb;
    int386(MOUSE_INT, &regs, &regs);
    mousedx = (int)((short)(regs.w.cx));
    mousedy = (int)((short)(regs.w.dx));

}

//--- Returns TRUE if a key has been hit since last call --------------------

char KeyHit(void)
{
    if (hit) {
        hit=FALSE;
        return TRUE;
    } else return FALSE;
}

//--- Returns TRUE if key has been hit and waits until it's released --------

char TestKey(int key)
{
    if (keys[key]) {
        while(keys[key]);
        hit=FALSE;
        return TRUE;
    } else return FALSE;
}

//--- Wait for key and return ascii code ------------------------------------

char GetChar(void) {
    char    ch;

    while (!hit);
     hit = 0;
    ch = transtab[lastcode];
    if (ch != 0) keys[lastcode] = 0;
    return(ch);
}

//--- Set mouse sensitivity -------------------------------------------------

void Set_MouseSensitivity(int s)
{
    union REGS regs;

    regs.x.eax = 0x1a;
    regs.x.ebx = s;
    regs.x.ecx = s;
    regs.x.edx = s;

    int386(MOUSE_INT, &regs, &regs);
}
