//****************************************************************************
//
// File         : input.cpp
//
// Description  : Linux-SVGALib/GCC keyboard/mouse routines...
//                Written by Lars Andersson (lars_and@algonet.se), 1998
//
//****************************************************************************

#define INPUT

#include <stdio.h>
#include <X11/Xlib.h>

#include "input.h"

extern Display 	*disp;

int		mousex, mousey, mousedx, mousedy, mousebuttons;
volatile char 	keys[256];
volatile    	char hit, lastcode, special;

 
//--- Init keyboard stuff ----------------------------------------------------

void Init_Keyboard(void)
{
   int i;

   for(i = 0; i < 256; i++) keys[i] = 0;
}

//--- Close down keyboard stuff ----------------------------------------------

void Close_Keyboard(void)
{

}

//--- Initialize mouse ------------------------------------------------------

int Init_Mouse(void)
{
return(TRUE);
}

void Close_Mouse(void)
{
 
}


//--- Update Input ------------------------------------------------------------

void Update_Input(void)
{
    int 	keycode, state;
    XEvent 	event;

    while(XCheckMaskEvent(disp, ~0, &event)) {
	keycode = FALSE;
	switch(event.type) {

    	    case ButtonPress : mousebuttons |= 1; break;

	    case KeyRelease : state = 0; keycode = TRUE; break;

	    case KeyPress : state = 1; keycode = TRUE; break;

            default: break;
        }
        if (keycode) {
            keycode = XLookupKeysym((XKeyEvent*)&event, 0);
	    special = (keycode >> 8) & 255;
	    keycode = keycode & 255;
//	    if ((keycode >= VERS_A_KEY) && (keycode <= VERS_Z_KEY)) 
//		keycode -= 0x20;

            if (state) {
		keys[keycode] = 1;
		hit = 1;
                lastcode = keycode; 
            }
            else {
    	        keys[keycode] = 0;
            }
	}    
    }
}


//--- Returns TRUE if a key has been hit since last call --------------------

char KeyHit(void)
{
    if (hit) {
        hit=FALSE;
        return TRUE;
    } else return FALSE;
}

//--- Returns TRUE if key has been hit and waits until it's released --------

char TestKey(int key)
{
    if (keys[key]) {
        while(keys[key]) Update_Input();
        return TRUE;
    } else return FALSE;
}

//--- Wait for key and return ascii code ------------------------------------

char GetChar(void) {

    unsigned char    ch;

    while (!hit) Update_Input();
    hit = 0;
    ch = lastcode;

    if ((ch >= 'a') && (ch <= 'z')) ch -= 32;
    else if (ch == BACKSPACE_KEY) ch = 8;
    else if (ch == RETURN_KEY) ch = 13;
    else if ((ch > 122) || special) ch = 0;

    return(ch);
}


//--- Set mouse sensitivity -------------------------------------------------

void Set_MouseSensitivity(int s, int d)
{
}
